

import 'package:origamicore/models/messageModel.dart';
import 'package:origamicore/models/packageIndex.dart';
import 'package:origamicore/models/serviceRoute.dart';
import 'package:origamicore/routing/routeResponse.dart';
import 'package:origamicore/routing/router.dart';

import 'models/databaseConfig.dart';
import 'models/databaseConnection.dart';
import 'models/databaseErrorMessage.dart';
import 'models/odataModel.dart';
import 'models/queryModel.dart';
import 'service/databaseIndex.dart';
import 'service/mongoIndex.dart';

class DatabasesIndex extends PackageIndex
{
  Map<String,DatabaseIndex> indexes={};
  List<DatabaseIndex> indexesArray=[];
  DatabaseConfig _cf=DatabaseConfig();
  
  @override
  Future jsonConfig(config) async { 
    Router.addRoute('database', ServiceRoute(services: {
      'search':search,
      'searchOne':searchOne,
      'save':save ,   
      'deleteOne':deleteOne ,   
      }));
    for(var a in config)
    {
      if(a['type']=='database')
      {
        _cf=DatabaseConfig.fromJson(a);
      }
    }
    
    for(var a in _cf.connections??[])
    {
      if(a.type==DatabaseType.MongoDb)
      {
        var index=MongoIndex(a);
        indexes[a.name??'']=index;
        indexesArray.add(index);
      }
    }
  } 

  @override
  void restart() {
    // TODO: implement restart
    
  }

  @override
  Future start() async { 
    for(var a in indexesArray)
    {
      await a.Connect();
      await a.Backup();
    }
  }

  @override
  void stop() {
    // TODO: implement stop
  }

  Future<RouteResponse> search(MessageModel message) async
  {
    String name=message.internalData['name'];
    if(name==null || !indexes.containsKey(name))
    { 
      return RouteResponse(error: DatabaseErrorMessage.connectionNotFound);
    } 
    var data= await indexes[name]?.Search(message.internalData['table'], QueryModel.fromJson(message.data['query']) , Odata());
    return RouteResponse(response:data );
  }
  Future<RouteResponse> searchOne(MessageModel message) async
  {
    String name=message.internalData['name'];
    if(name==null || !indexes.containsKey(name))
    { 
      return RouteResponse(error: DatabaseErrorMessage.connectionNotFound);
    } 
    var data= await indexes[name]?.SearchOne(message.internalData['table'], QueryModel.fromJson(message.data['query']) );
    return RouteResponse(response:data );
  }
  Future<RouteResponse> save(MessageModel message) async
  {
    String name=message.internalData['name'];
    if(name==null || !indexes.containsKey(name))
    { 
      return RouteResponse(error: DatabaseErrorMessage.connectionNotFound);
    } 
    var data= await indexes[name]?.Save(message.internalData['table'],  message.data['keys'], message.data['object']);
    return RouteResponse(response:data );
  }
  Future<RouteResponse> deleteOne(MessageModel message) async
  {
    String name=message.internalData['name'];
    if(name==null || !indexes.containsKey(name))
    { 
      return RouteResponse(error: DatabaseErrorMessage.connectionNotFound);
    } 
    var data= await indexes[name]?.DeleteOne(message.internalData['table'],  message.data['keys'], message.data['object']);
    return RouteResponse(response:data );
  }

}