import 'dart:async';
import 'dart:io';

import 'package:mongo_dart/mongo_dart.dart';
import 'package:origami_database/models/actionModel.dart';
import 'package:origamicore/routing/routeResponse.dart';
import 'package:process_run/shell.dart';
import '../models/odataModel.dart';
import '../models/queryModel.dart';

import '../models/databaseConnection.dart';

import 'databaseIndex.dart';
 

class MongoIndex extends DatabaseIndex
{ 
  Db db;
  DatabaseConnection connection;
  MongoIndex(this.connection);
  @override
  Future<void> Connect()async {
    var user='';
    if(connection.username!='')
    {
      user='${connection.username}:${connection.password}@';
    }
    var con = 'mongodb://$user${connection.host}:${connection.port.toString()}/${connection.database}';
    db = await Db(con);
    await db.open();
    print('database connect');
  }  
  @override
  Future<ResponseDataModel> Search(String table, QueryModel model, Odata odata)async {
    var coll = db.collection(table); 
    var find=  coll.find( model.where);
    // var x=where.gt("my_field", 995); 
    var w=where;  
     w.map['\$query']= model.where??<String,dynamic>{};
    
    if(model.top!=null)
    {  
      w.limit(model.top); 
    }
    if(model.skip!=null)
    {
      w.skip(model.skip);
    }
    if(model.sort.isNotEmpty)
    {
      for(var sort in model.sort)
      {
        w.sortBy(sort.prop,descending: (sort.isDesc??false));
      }
    }
    var data=await coll.find( w).toList();      
    return ResponseDataModel(data:{'value':data},isDone:true);
  }

  @override
  Future<ResponseDataModel> SearchOne(String table, QueryModel model) async  { 
    var coll = db.collection(table);
    var dt =await coll.findOne(model.where);
    //  coll.find(WhereCondition.getWhere(model.where,where)).toList();
    return ResponseDataModel(data:dt,isDone:true);//
  }

  @override
  Future<ResponseDataModel> Delete(String table, QueryModel model) {
      // TODO: implement Delete
      throw UnimplementedError();
    }
  
    @override
    Future<ResponseDataModel> DeleteOne(String table, List<String> keys, data)async {
      var key=<String,dynamic>{};
      for(var a in keys)
      {
        key[a]=data[a];
      }
      var coll = db.collection(table);
      var deletedData =  await coll.remove(key);
      return ResponseDataModel(data:deletedData,isDone: true );
    }
  
    @override
    Future<ResponseDataModel> Save(String table, List<String> keys,dynamic data)async {
    var key=<String,dynamic>{};
    for(var a in keys)
    {
      key[a]=data[a];
    }
    var coll = db.collection(table);
    var dt = await coll.findOne(key);
    if(dt==null)
    {
      dt=data;
    }
    else
    {
      data.forEach((key, value) { 
        dt[key]=value;
      });
    }
    dynamic obj={};
      obj=await coll.update(key,dt,upsert: true);
    var isSucces=false;
    if(obj['ok']!=null)
    {
      isSucces=true;
    }
    return ResponseDataModel(data:DatabaseActionModel(key:key,success:isSucces).toJson());
  }

  @override
  Future<void> Backup()async {
    var backup=connection.backup;
    if(backup==null)
    {
      print('backup not exist');
      return;
    }
    var time=0;
    var filename='./${connection.name}.txt';
    if(FileSystemEntity.typeSync(filename)==FileSystemEntityType.notFound)
    {
      time=DateTime.now().millisecondsSinceEpoch;
      File(filename).writeAsStringSync(time.toString());
    }
    else
    {
      time = int.parse(File(filename).readAsStringSync());
    }
    Timer.periodic(Duration(seconds: 10), (timer)async {
      var shell = Shell();
      var now=DateTime.now().millisecondsSinceEpoch;
      if(now>time)
      {
        print('start backup');
        time=DateTime.now().millisecondsSinceEpoch+ 1000*backup.secPeriod;
        var directory=backup.path+DateTime.now().millisecondsSinceEpoch.toString();
        await   Directory(directory).create();
        if(backup.tables.isEmpty)
        {
          await shell.run('mongodump --out=${directory} --db=${connection.database}');
        }
        else
        { 
          var collections=backup.tables.split(',');
          for(var a in collections)
          {
           var data= await shell.run('mongodump --out=${directory} --db=${connection.database} --collection=${a}');
           var x=0;
          }
        }
        var a=0;
        File(filename).writeAsStringSync(time.toString());
      }
     });
  }


}
class WhereCondition
{
  static SelectorBuilder getWhere(dynamic data,SelectorBuilder where)
  {
    return SelectorBuilder();
  }
}